<?php
/*
Plugin Name: MSTW Bracket Builder
Plugin URI: https://wordpress.org/plugins/mstw-bracket-builder/
Description: Builds brackets for 4 to 64 team tournaments.
Version: 1.3
Author: Mark O'Donnell
Author URI: http://shoalsummitsolutions.com
Text Domain: mstw-bracket-builder
*/

/*---------------------------------------------------------------------------
 *	MSTW Wordpress Plugins (http://shoalsummitsolutions.com)
 *	Copyright 2016-22 Mark O'Donnell (mark@shoalsummitsolutions.com)
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *	GNU General Public License for details. <http://www.gnu.org/licenses/>
 *-------------------------------------------------------------------------*/

//------------------------------------------------------------------------
// Initialize the plugin ... include files, define globals, register CPTs
//
add_action( 'init', 'mstw_bb_init' );

function mstw_bb_init( ) {
	//mstw_bb_log_msg( "mstw_bb_init:" );
	//------------------------------------------------------------------------
	// "Helper functions" used throughout MSTW Schedule Builder
	//
	require_once( plugin_dir_path( __FILE__ ) . 'includes/mstw-bb-utility-functions.php' );
	
	//-----------------------------------------------------------------
	// Include the necessary files for admin screens
	//
	
	require_once( plugin_dir_path( __FILE__ ) . 'includes/mstw-bb-cpts.php'  );
	mstw_bb_register_cpts( );
	
	// ----------------------------------------------------------------
	// Set up translation/localization with the 'mstw-bracket-builder'
	// This means the WP language packs will be used exclusively
	//
	load_plugin_textdomain( 'mstw-bracket-builder' );
	
	//-----------------------------------------------------------------
	// If on an admin screen, load the admin functions (gotta have 'em)
	//
	if ( is_admin( ) ) {
		require_once ( plugin_dir_path( __FILE__ ) . 'includes/mstw-bb-admin.php' );
	}
	
	// ----------------------------------------------------------------
	// add ajax action for all bracket builder screen
	// mstw_bb_ajax_callback( ) is in mstw-bb-admin.php
	//
	add_action( 'wp_ajax_bracket_builder', 'mstw_bb_ajax_callback' );
	
	
	// Enqueue the plugin's stylesheet and any scripts
	add_action( 'wp_enqueue_scripts', 'mstw_bb_enqueue_scripts' );
	
	//------------------------------------------------------------------------
	// Functions for MSTW Tourney Bracket shortcode
	//
	include_once( plugin_dir_path( __FILE__ ) . 'includes/mstw-bb-tourney-bracket-class.php' );
	
	
	//------------------------------------------------------------------------
	// Functions for MSTW venue table shortcode
	//
	include_once( plugin_dir_path( __FILE__ ) . 'includes/mstw-bb-tourney-table-class.php' );
	
} //End: mstw_bb_init( )

// ----------------------------------------------------------------
// filter so single-tourney template does  not need to be in the theme directory
//
add_filter( "single_template", "mstw_bb_single_tourney_template" );

function mstw_bb_single_tourney_template( $single_template ) {
	//mstw_bb_log_msg( "mstw_bb_single_tourney_template" );
	 global $post;

	 if ($post->post_type == 'mstw_bb_tourney') {
		  $single_template = dirname( __FILE__ ) . '/templates/single-tourney.php';  
	 }
	
	 return $single_template;
	 
} //End: mstw_bb_single_tourney_template()

// ----------------------------------------------------------------
// Enqueue the stylesheet for the front end
// add_filter( 'enqueue_styles', 'mstw_bb_enqueue_styles ) is in mstw_bb_init( )
//
function mstw_bb_enqueue_scripts( ) {
	//mstw_bb_log_msg( "mstw_bb_enqueue_scripts:" );
	
	// Find the full path to the plugin's css file 
	$mstw_bb_style_url = plugins_url( '/css/mstw-bb-styles.css', __FILE__ );
	
	wp_register_style( 'mstw_bb_style', $mstw_bb_style_url );
	
	$mstw_bb_style_file = plugin_dir_path( __FILE__ ) . 'css/mstw-bb-styles.css';

	// If stylesheet exists, enqueue the style
	if ( file_exists( $mstw_bb_style_file ) ) {	
		wp_enqueue_style( 'mstw_bb_style' );			
	} 

	$mstw_bb_custom_stylesheet = get_stylesheet_directory( ) . '/mstw-bb-custom-styles.css';
	
	if ( file_exists( $mstw_bb_custom_stylesheet ) ) {
		$mstw_bb_custom_stylesheet_url = get_stylesheet_directory_uri( ) . '/mstw-bb-custom-styles.css';
		wp_register_style( 'mstw_bb_custom_style', $mstw_bb_custom_stylesheet_url );
		wp_enqueue_style( 'mstw_bb_custom_style' );
	}
	
} //End: mstw_bb_enqueue_scripts( )
?>